package com.gic.auth.web.controller;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.gic.auth.web.vo.CollaboratorDetailVO;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.constant.MenuLevelConstants;
import com.gic.auth.constant.MenuProjectConstants;
import com.gic.auth.dto.MenuDTO;
import com.gic.auth.qo.CollaboratorListQO;
import com.gic.auth.qo.MenuListQO;
import com.gic.auth.service.CollaboratorApiService;
import com.gic.auth.service.MenuApiService;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.enterprise.response.EnterpriseRestResponse;
import com.gic.enterprise.utils.ResultControllerUtils;
import com.gic.enterprise.utils.UserDetailUtils;

@RestController
@RequestMapping("/collaborator")
public class CollaboratorController {
    private static final Logger LOGGER = LogManager.getLogger(CollaboratorController.class);

    @Autowired
    private CollaboratorApiService collaboratorApiService;
    @Autowired
    private MenuApiService menuApiService;

    @RequestMapping("/list")
    public RestResponse list(CollaboratorListQO params) {
        params.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        return ResultControllerUtils.commonResult(collaboratorApiService.page(params));
    }

    @RequestMapping("/list-app")
    public RestResponse listApp() {
        return ResultControllerUtils
                .commonResult(collaboratorApiService.listApp(UserDetailUtils.getUserDetail().getEnterpriseId()));
    }

    @RequestMapping("/list-sub-app")
    public RestResponse listSubApp(String appId) {
        return ResultControllerUtils.commonResult(
                collaboratorApiService.listSubApp(appId, UserDetailUtils.getUserDetail().getEnterpriseId()));
    }

    @RequestMapping("/get-detail")
    public RestResponse getDetail(Integer collaboratorId) {
        return ResultControllerUtils.commonResultOne(
                collaboratorApiService.getDetail(collaboratorId), CollaboratorDetailVO.class);
    }

    /**
     * 授权：操作授权、资源授权
     * @Title: auth 
     * @Description:
      * @author guojuxing
     * @param collaboratorId 
     * @return  com.gic.commons.webapi.reponse.RestResponse 
      */
    @RequestMapping("/auth")
    public RestResponse auth(String subAppIds, String menuIds, Integer collaboratorId) {
        return ResultControllerUtils.commonResult(
                collaboratorApiService.auth(subAppIds, menuIds, collaboratorId));
    }

    @RequestMapping("/app-menu-tree")
    public RestResponse appMenuTree(String appId) {
        MenuListQO params = new MenuListQO();
        params.setProject(appId);
        params.setIsGIC(MenuProjectConstants.APP_PROJECT_CODE);
        ServiceResponse<List<MenuDTO>> appMenuList = menuApiService.listMenuTree(params);
        if (appMenuList.isSuccess()) {
            return RestResponse.success(changePageListToTree(MenuLevelConstants.FIRST_LEVEL_PARENT_ID, appMenuList.getResult()));
        }
        return EnterpriseRestResponse.failure(appMenuList);
    }

    public static List<Map<String, Object>> changePageListToTree(int storeGroupId, List<MenuDTO> list) {
        List<Map<String, Object>> result = new ArrayList<>();
        if (list != null) {
            for (int j = 0, length = list.size(); j < length; j++) {
                MenuDTO menuDTO = list.get(j);
                if (storeGroupId == menuDTO.getParentId()) {
                    Map<String, Object> data = new HashMap<>(16);
                    data.put("menuId", menuDTO.getMenuId());
                    data.put("menuName", menuDTO.getMenuName());
                    data.put("menuCode", menuDTO.getMenuCode());
                    data.put("parentCode", menuDTO.getParentCode());
                    data.put("parentId", menuDTO.getParentId());
                    List<Map<String, Object>> children = changePageListToTree(menuDTO.getMenuId(), list);
                    if (children == null || children.size() < 1) {
                        data.put("children", null);
                    } else {
                        data.put("children", children);
                    }
                    result.add(data);
                }
            }
        }
        return result;
    }
}
